/*
 *
 * SOC Robotics, Inc
 *
 * File - 
 *
 * Compiler - WIN AVR 20080610
 * IDE		- AVR Studio 4.15
 * 
 *
 * Description:
 *
 * Revision: 1.01 -> Date: December 9th, 2008			BR
 * Revision: 1.02 -> Date: April 6th, 2010				SS
 *	>	
 *
 *
 *
 * History:
 * 
 *
 * Copyright 2010, SOC Robotics, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 * 4. Redistribution is only allowed in combination with SOC ROBOTICS, 
 *    Inc. hardware products.
 *
 * THIS SOFTWARE IS PROVIDED BY SOC Robotics, Inc. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SOC ROBOTICS, 
 * INC. OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.soc-robotics.com/
 */

 /*
             MyUSB Library
     Copyright (C) Dean Camera, 2008.

  dean [at] fourwalledcubicle [dot] com
      www.fourwalledcubicle.com
*/

/*
  Copyright 2008  Dean Camera (dean [at] fourwalledcubicle [dot] com)

  Permission to use, copy, modify, and distribute this software
  and its documentation for any purpose and without fee is hereby
  granted, provided that the above copyright notice appear in all
  copies and that both that the copyright notice and this
  permission notice and warranty disclaimer appear in supporting
  documentation, and that the name of the author not be used in
  advertising or publicity pertaining to distribution of the
  software without specific, written prior permission.

  The author disclaim all warranties with regard to this
  software, including all implied warranties of merchantability
  and fitness.  In no event shall the author be liable for any
  special, indirect or consequential damages or any damages
  whatsoever resulting from loss of use, data or profits, whether
  in an action of contract, negligence or other tortious action,
  arising out of or in connection with the use or performance of
  this software.
*/

/*
	Communications Device Class demonstration application.
	This gives a simple reference application for implementing
	a USB to Serial converter using the CDC class. Sent and
	recieved data on the serial port is communicated to the USB
	host.

	Before running, you will need to install the INF file that
	is located in the USBtoSerial project directory. This will enable
	Windows to use its inbuilt CDC drivers, negating the need
	for special Windows drivers for the device.
*/

/*
	USB Mode:           Device
	USB Class:          Communications Device Class (CDC)
	USB Subclass:       Abstract Control Model (ACM)
	Relevant Standards: USBIF CDC Class Standard
	Usable Speeds:      Full Speed Mode
*/

#include "USB10.h"
#include "I2C_bitbanged.h"
#include "StepDirLineCoding.h"
#include <avr/pgmspace.h> 


#define F_INTERNAL 8000000UL

/** Macro for calculating the baud value from a given baud rate when the U2X (double speed) bit is
 *  not set.
 */
#define MY_SERIAL_UBBRVAL(baud)    (((F_INTERNAL / 16) / baud) - 1)

/** Macro for calculating the baud value from a given baud rate when the U2X (double speed) bit is
 *  set.
 */
#define MY_SERIAL_2X_UBBRVAL(baud) (((F_INTERNAL / 8) / baud) - 1)


#define COMMAND 0
#define SERIAL 1
#define ISP1 2
#define ISP2 3


#define MISO_2 7
#define MOSI_2 4
#define SCK_2 5
#define RES_2 2

#define INTERBITDELAY 10
#define HIGH 1
#define LOW 0


#define greenLedOn() PORTB &= ~(1<<5)
#define greenLedOff() PORTB |= 1<<5
#define redLedOn() PORTC &= ~(1<<2)
#define redLedOff() PORTC |= 1<<2

void StringToBuffer_P(const char *data);
void ISP_Update_Buffers(void);
void ISP_Int(void);

void  Write_ISP_Byte(unsigned char outbyte);


unsigned char Write_Read_ISP_Byte(unsigned char outbyte);
unsigned char Wait_For_Busy(void);
unsigned char Chip_Busy_Status(void);
void Change_ISP_Pin(unsigned char pinId,unsigned char state);
void DelayUsec(unsigned char delay);

void (*bootloader) (void)=(void (*)(void))(0x1800);


/* Scheduler Task List */
TASK_LIST
{
	{ Task: USB_USBTask          , TaskStatus: TASK_STOP },
	{ Task: CDC_Task             , TaskStatus: TASK_STOP },
	{ Task: Command_Task         , TaskStatus: TASK_STOP },
	{ Task: PROG_Task            , TaskStatus: TASK_STOP },
};

/* Globals: */
CDC_Line_Coding_t LineCoding = { BaudRateBPS: 9600,
                                 CharFormat:  OneStopBit,
                                 ParityType:  Parity_None,
                                 DataBits:    8            };

volatile RingBuff_t Rx_Buffer;
volatile RingBuff_t Tx_Buffer;

volatile bool Transmitting = false;
volatile bool escapeCharReceived = false;

uint8_t mode = COMMAND;
//uint8_t mode = SERIAL;

uint8_t setUpM = 0;//used to check to see if we need to setup our port directions
uint8_t setUpLed = 0;

const char MYSTRING[] PROGMEM = "I'm in Flash";


int main(void)
{
	/* Disable watchdog if enabled by bootloader/fuses */
	MCUSR &= ~(1 << WDRF);
	wdt_disable();

    // For 16MHz operation the external crystal must be divided by 1
	/* Divide the 16Mhz crystal clock to 8Mhz to allow for 3.3V operation*/
	SetSystemClockPrescaler(1);

	/* Don't divide the clock if 8Mhz crystal is installed 8Mhz to allow for 3.3V operation*/
//	SetSystemClockPrescaler(0);

	/* Ringbuffer Initialization */
	Buffer_Initialize(&Rx_Buffer);
	Buffer_Initialize(&Tx_Buffer);


	/* Initialize Scheduler so that it can be used */
	Scheduler_Init();

	/* Initialize USB Subsystem */
	USB_Init();

	
	/* Scheduling - routine never returns, so put this last in the main function */
	Scheduler_Start();
}


EVENT_HANDLER(USB_Connect)
{
	/* Start USB management task */
	Scheduler_SetTaskMode(USB_USBTask, TASK_RUN);

}

EVENT_HANDLER(USB_Disconnect)
{
	/* Stop running CDC and USB management tasks */
	Scheduler_SetTaskMode(CDC_Task, TASK_STOP);
	Scheduler_SetTaskMode(USB_USBTask, TASK_STOP);

}

EVENT_HANDLER(USB_ConfigurationChanged)
{
	/* Setup CDC Notification, Rx and Tx Endpoints */
	Endpoint_ConfigureEndpoint(CDC_NOTIFICATION_EPNUM, EP_TYPE_INTERRUPT,
		                       ENDPOINT_DIR_IN, CDC_NOTIFICATION_EPSIZE,
	                           ENDPOINT_BANK_SINGLE);

	Endpoint_ConfigureEndpoint(CDC_TX_EPNUM, EP_TYPE_BULK,
		                       ENDPOINT_DIR_IN, CDC_TXRX_EPSIZE,
	                           ENDPOINT_BANK_DOUBLE);

	Endpoint_ConfigureEndpoint(CDC_RX_EPNUM, EP_TYPE_BULK,
		                       ENDPOINT_DIR_OUT, CDC_TXRX_EPSIZE,
	                           ENDPOINT_BANK_DOUBLE);


	/* Start CDC task */
	Scheduler_SetTaskMode(CDC_Task, TASK_RUN);
}

EVENT_HANDLER(USB_UnhandledControlPacket)
{
	uint8_t* LineCodingData = (uint8_t*)&LineCoding;

	/* Process CDC specific control requests */
	switch (bRequest) 	{

		case GET_LINE_CODING:
			if (bmRequestType == (REQDIR_DEVICETOHOST | REQTYPE_CLASS | REQREC_INTERFACE)) {
				/* Acknowedge the SETUP packet, ready for data transfer */
				Endpoint_ClearSetupReceived();

				/* Write the line coding data to the control endpoint */
				Endpoint_Write_Control_Stream_LE(LineCodingData, sizeof(LineCoding));

				/* Send the line coding data to the host and clear the control endpoint */
				Endpoint_ClearSetupOUT();
			    }
			break;

		case SET_LINE_CODING:
			if (bmRequestType == (REQDIR_HOSTTODEVICE | REQTYPE_CLASS | REQREC_INTERFACE)) {
				/* Acknowedge the SETUP packet, ready for data transfer */
				Endpoint_ClearSetupReceived();

				/* Read the line coding data in from the host into the global struct */
				Endpoint_Read_Control_Stream_LE(LineCodingData, sizeof(LineCoding));

				/* Send the line coding data to the host and clear the control endpoint */
				Endpoint_ClearSetupIN();

				/* Reconfigure the USART with the new settings */
				ReconfigureUSART();
			    }
			break;

		case SET_CONTROL_LINE_STATE:
			if (bmRequestType == (REQDIR_HOSTTODEVICE | REQTYPE_CLASS | REQREC_INTERFACE)) {
				/* Acknowedge the SETUP packet, ready for data transfer */
				Endpoint_ClearSetupReceived();

				/* Send an empty packet to acknowedge the command (currently unused) */
				Endpoint_ClearSetupIN();
				if (mode != SERIAL)    {//in case we were in SPI mode
					mode = COMMAND;
					Buffer_Initialize(&Rx_Buffer);//clear my buffers
					Buffer_Initialize(&Tx_Buffer);
					
					Scheduler_SetTaskMode(CDC_Task, TASK_RUN);
					Scheduler_SetTaskMode(PROG_Task, TASK_STOP);
				    }

				if(mode == COMMAND) {
					StringToBuffer_P(PSTR("\r\nUSB10 Ferret Utility V1.02\n\r(c) 2010 SOC Robotics, Inc.\r\n\r\n"));
					Buffer_StoreElementOnTop(&Tx_Buffer, '>');//prompt character
							//mode = SERIAL;

/*			// Release PB0 - make input
			DDRB &= ~(1<<0);
			// Set Reset High - PC2
			DDRC |= (1<<2);
			PORTC |= (1<<2);

			if(Buffer_GetElementFromBottom(&Rx_Buffer) == 'b')  {//bang the reset line
				DDRC |= (1<<RES_2);
				PORTC &= ~(1<<RES_2);
				DelayUsec(100);
				PORTC |= (1<<RES_2);
			    }
			Buffer_Initialize(&Rx_Buffer);//dump any unused data
			Buffer_Initialize(&Tx_Buffer);//dump any unused data
			break;

*/
				    }
				
			    }
			break;
	}
}

void StringToBuffer_P(const char *data)
{
	while (pgm_read_byte(data) != 0x00)
	{
		Buffer_StoreElementOnTop(&Tx_Buffer, pgm_read_byte(data++));
	}

}


TASK(CDC_Task)
{
	uint8_t temp;
	
	if (USB_IsConnected)
	{
		/* Select the Serial Rx Endpoint */
		Endpoint_SelectEndpoint(CDC_RX_EPNUM);

		if (Endpoint_ReadWriteAllowed())
		{
			/* Read the recieved data endpoint into the transmission buffer */
			while (Endpoint_BytesInEndpoint())
			{
				if(mode == COMMAND)
				{
					temp =  Endpoint_Read_Byte();

					Buffer_StoreElementOnTop(&Tx_Buffer, temp);//echo the character back
					Buffer_StoreElementOnTop(&Rx_Buffer, temp);//save it to our buffer

					if(temp == '\b')
					{
						Buffer_StoreElementOnTop(&Tx_Buffer, ' ');//clear the mistake
						Buffer_StoreElementOnTop(&Tx_Buffer, '\b');
					
						Buffer_GetElementFromTop(&Rx_Buffer);//remove the \b and the character sent from our buffer
						Buffer_GetElementFromTop(&Rx_Buffer);
					}
					else if(temp == '\r')
					{
					//	Buffer_StoreElementOnTop(&Tx_Buffer, '\n');//ensure \r\n is sent back
						Buffer_GetElementFromTop(&Tx_Buffer);//remove the \r
						Buffer_GetElementFromTop(&Rx_Buffer);//remove the \r

					
						Scheduler_SetTaskMode(CDC_Task, TASK_STOP);//stop it for a while


						Scheduler_SetTaskMode(Command_Task, TASK_RUN);//process the data

						break;//don't care about any other data
					}
				}
				else if(mode == SERIAL)
				{
					temp = Endpoint_Read_Byte();
					if(temp == '\\')
					{
						if (escapeCharReceived == true)
						{

							escapeCharReceived = false;
							Buffer_StoreElementOnTop(&Rx_Buffer, '\\');//pass on the old one
							Buffer_StoreElementOnTop(&Rx_Buffer, temp);//pass on the new one
						}
						else
						{
							escapeCharReceived = true;
						}
					}
					else if( escapeCharReceived == true)
					{
					
						if(temp == 'c' || temp == 'C')
						{
							/* Initiate the transmission of the buffer contents if USART idle */
							if (Transmitting == false && Rx_Buffer.Elements != 0)
							{
								Transmitting = true;
								Serial_TxByte(Buffer_GetElementFromBottom(&Rx_Buffer));
							}
							while(Transmitting == true)//wait until we finish
							{
								;
							}
							mode = COMMAND;
							Buffer_Initialize(&Rx_Buffer);//clear out all data
							Buffer_Initialize(&Tx_Buffer);

							Buffer_StoreElementOnTop(&Tx_Buffer, '\r');
							Buffer_StoreElementOnTop(&Tx_Buffer, '\n');
							Buffer_StoreElementOnTop(&Tx_Buffer, '>');

						}
						else
						{
							Buffer_StoreElementOnTop(&Rx_Buffer, '\\');//pass on all the data
							Buffer_StoreElementOnTop(&Rx_Buffer, temp);
						}

						escapeCharReceived = false;
					}
					else
					{
						Buffer_StoreElementOnTop(&Rx_Buffer, temp);
					}	
				}
			}

			/* Clear the endpoint buffer */
			Endpoint_ClearCurrentBank();
		}
		

		/* Select the Serial Tx Endpoint */
		Endpoint_SelectEndpoint(CDC_TX_EPNUM);

		/* Initiate the transmission of the buffer contents if USART idle */
		if (Transmitting == false && mode == SERIAL && Rx_Buffer.Elements != 0)
		{
				Transmitting = true;
				Serial_TxByte(Buffer_GetElementFromBottom(&Rx_Buffer));
		}


		/* Write the Data to the Endpoint */
		if(Tx_Buffer.Elements != 0)
		{

			do{
				/* Wait until Serial Tx Endpoint Ready for Read/Write */
				while (!(Endpoint_ReadWriteAllowed()));

				Endpoint_Write_Byte(Buffer_GetElementFromBottom(&Tx_Buffer));

				/* Check if the endpoint is full */
				if (Endpoint_BytesInEndpoint() == CDC_TXRX_EPSIZE)
				{
					/* Send the data */
					Endpoint_ClearCurrentBank();

					/* Wait until Serial Tx Endpoint Ready for Read/Write */
					while (!(Endpoint_ReadWriteAllowed()));
				}
			}while (Tx_Buffer.Elements != 0);

			/* Send the last packet of the transfer to the host */
			Endpoint_ClearCurrentBank();

		}
	}
}



TASK(Command_Task)
{
	uint8_t i, address = 0, lenght = 0, axis, oldValue,baudrate,hundreds,tens,ones;
	uint8_t temp[64];


	RemoveWhiteSpace();


	switch (Buffer_GetElementFromBottom(&Rx_Buffer))
	{		
		case 'p':

			oldValue = Buffer_GetElementFromBottom(&Rx_Buffer);
			
			if(oldValue == '1')
			{
				mode = ISP1;
				
			}
			else
			{
				mode = ISP2;
			}
			
			ISP_Int();

			Buffer_Initialize(&Rx_Buffer);//dump any unused data
			Buffer_Initialize(&Tx_Buffer);//dump any unused data

			break;
		
		
		case 'c':
		case 'C':
			
			I2C_Port = Buffer_GetElementFromBottom(&Rx_Buffer);
			//should check to ensure val ok!
			
			I2C_Init();

			RemoveWhiteSpace();

			address = ParseNumber();
			RemoveWhiteSpace();

			oldValue = Buffer_GetElementFromBottom(&Rx_Buffer);

			if(oldValue == 'w' || oldValue == 'W')
			{
						
				lenght = ParseNumber();

				oldValue = Buffer_GetElementFromBottom(&Rx_Buffer);
				if(oldValue != ' ' && oldValue != ',')
				{
					Buffer_StoreElementOnBottom(&Rx_Buffer, oldValue);
				}
						
				for (i = 0; i < lenght; i++)
				{
					temp[i] = Buffer_GetElementFromBottom(&Rx_Buffer);
					if(temp[i] == '\\')
					{	
						oldValue = Buffer_GetElementFromBottom(&Rx_Buffer);

						if(oldValue == 'x' || oldValue == 'X')
						{
							temp[i] = GetHexNumber();
						}
						else
						{
							if(i < lenght-1)//will we even get a chance to send it.  If so write it
							{
								temp[++i] = oldValue;
							}
							else//put the data back.  This is not needed though as we will dump the data latter
							{
								Buffer_StoreElementOnBottom(&Rx_Buffer, oldValue);
							} 
						}	
					}
				}

				if(I2C_Write(address, temp, lenght) != lenght)
				{
					Buffer_StoreElementOnTop(&Tx_Buffer, '\r');
					Buffer_StoreElementOnTop(&Tx_Buffer, '\n');
					Buffer_StoreElementOnTop(&Tx_Buffer, '-');
				}					
						 
			}
			else if(oldValue == 'r' || oldValue == 'R')
			{
				Buffer_StoreElementOnTop(&Tx_Buffer, '\r');//so the data read back will be printed on a new line
				Buffer_StoreElementOnTop(&Tx_Buffer, '\n');

				lenght = ParseNumber();

				if(I2C_Read(address, temp, lenght) == lenght)
				{
					for (i = 0; i < lenght; i++)
					{
						Buffer_StoreElementOnTop(&Tx_Buffer, temp[i]);
					}
				}
				else
				{
					Buffer_StoreElementOnTop(&Tx_Buffer, '\r');
					Buffer_StoreElementOnTop(&Tx_Buffer, '\n');
					Buffer_StoreElementOnTop(&Tx_Buffer, '-');

				}			
			}
			
			break;
		case '?':
			StringToBuffer_P(PSTR("\r\nUSB10 Command Summary\r\n"));

			/*
			StringToBuffer_P(PSTR("UBRR1L="));
			baudrate = UBRR1L;
			hundreds = baudrate/100;
			tens = (baudrate - hundreds*100)/10;
			ones = (baudrate - hundreds*100 - tens*10);
		    Buffer_StoreElementOnTop(&Tx_Buffer, hundreds+0x30);
		    Buffer_StoreElementOnTop(&Tx_Buffer, tens+0x30);
		    Buffer_StoreElementOnTop(&Tx_Buffer, ones+0x30);
			StringToBuffer_P(PSTR("  "));
			StringToBuffer_P(PSTR("UBRR1H="));
		    Buffer_StoreElementOnTop(&Tx_Buffer, UBRR1H+0x30);
			StringToBuffer_P(PSTR("\r\n"));
			*/

			StringToBuffer_P(PSTR(" s-serial,c-I2C,p-prog,m-motor,l-led,b-boot\r\n"));
			break;
		case 'm':
		case 'M':

			if(setUpM == 0)
			{
				StepDirLineCoding_Init();
				setUpM = 1;
			}	
			RemoveWhiteSpace();

			switch(Buffer_GetElementFromBottom(&Rx_Buffer))
			{
				case 'y':
				case 'Y':
					axis = Y_AXIS;
					break;

				case 'z':
				case 'Z':
					axis = Z_AXIS;
					break;
				
				case 'x':
				case 'X':
				default:
					axis = X_AXIS;
					break;
				
			}

			RemoveWhiteSpace();

			lenght = ParseNumber();

			oldValue = Buffer_GetElementFromBottom(&Rx_Buffer);

			if(oldValue != ' ' && oldValue != ',')//allow for 1 peice of white space
			{
				Buffer_StoreElementOnBottom(&Rx_Buffer, oldValue);
			}
						
			for (i = 0; i < lenght && Rx_Buffer.Elements != 0; i++)
			{
				SendByte(Buffer_GetElementFromBottom(&Rx_Buffer),axis);
			}

			break;

		case 's':  // Enter serial mode
		case 'S':
			
			mode = SERIAL;

			// Work around for baud rate calculation failing for numbers bigger
			// than 64K
			if(UBRR1L==19) UBRR1L = 8;  //  115,200 baud case
			else if(UBRR1L==28) UBRR1L = 4;  // 230,400 baud case


			// Release PB0 - make input
			DDRB &= ~(1<<0);
			// Set Reset High - PC2
			DDRC |= (1<<2);
			PORTC |= (1<<2);

			if(Buffer_GetElementFromBottom(&Rx_Buffer) == 'b')//bang the reset line
			{

				DDRC |= (1<<RES_2);

				PORTC &= ~(1<<RES_2);
				DelayUsec(100);
				PORTC |= (1<<RES_2);
			}
			Buffer_Initialize(&Rx_Buffer);//dump any unused data
			Buffer_Initialize(&Tx_Buffer);//dump any unused data
			break;

		case 0://empty command
			break;

		case 'b'://start the bootloader
		case 'B':
			(*bootloader)();//will not return
			break;

		case 'l':
		case 'L':

			// Setup LED port pins and turn leds off
			if(setUpLed == 0) {
				setUpLed = 1;
				DDRC |= (1<<2);
				PORTC |= (1<<2);
				DDRB |= (1<<5);
				PORTB |= (1<<5);
				}

			RemoveWhiteSpace();

			switch(Buffer_GetElementFromBottom(&Rx_Buffer))
			{
				case 'g':
				case 'G':
						
					RemoveWhiteSpace();

					switch(Buffer_GetElementFromBottom(&Rx_Buffer))
					{
						case 'o':
						case 'O':
							greenLedOn();
							break;

						case 'f':
						case 'F':
							greenLedOff();				
							break;
						default:
							break;
					}
					
					break;

				case 'r':
				case 'R':


					RemoveWhiteSpace();

					switch(Buffer_GetElementFromBottom(&Rx_Buffer))
					{
						case 'o':
						case 'O':
							redLedOn();
							break;

						case 'f':
						case 'F':
							redLedOff();				
							break;
						default:
							break;
				
					}
					break;

				default:
					break;
				
			}

			break;

		default:
			Buffer_StoreElementOnTop(&Tx_Buffer, '\r');
			Buffer_StoreElementOnTop(&Tx_Buffer, '\n');
			Buffer_StoreElementOnTop(&Tx_Buffer, '?');			

	}
	


	
	switch(mode)
	{
		case ISP1:
		case ISP2:	
			Buffer_StoreElementOnTop(&Tx_Buffer, 'a');
			Scheduler_SetTaskMode(PROG_Task, TASK_RUN);
			break;

		case SERIAL:
			Scheduler_SetTaskMode(CDC_Task, TASK_RUN);
			break;

		default://Return a <CR>
			Buffer_StoreElementOnTop(&Tx_Buffer, '\r');
			Buffer_StoreElementOnTop(&Tx_Buffer, '\n');
			Buffer_StoreElementOnTop(&Tx_Buffer, '>');
					
			Buffer_Initialize(&Rx_Buffer);//dump any unused data
			Scheduler_SetTaskMode(CDC_Task, TASK_RUN);
	}

	Scheduler_SetTaskMode(Command_Task, TASK_STOP);

}




TASK(PROG_Task)
{
	unsigned char repeat;
	
	ISP_Update_Buffers();

	do
	{
		repeat = 0;

		if(Rx_Buffer.Elements != 0)
		{
			switch(Buffer_GetElementFromBottom(&Rx_Buffer))
			{
				case 'r':
					if (Rx_Buffer.Elements)//ensure we have received enough data
					{
						if(Buffer_GetElementFromBottom(&Rx_Buffer) == 'l')//set reset low on target board
						{
							if(mode == ISP2)
							{
								Change_ISP_Pin(RES_2,LOW);
							}
							else
							{
								;
							}
						}
						else//set reset high
						{
							if(mode == ISP2)
							{
								Change_ISP_Pin(RES_2, HIGH);
							}
							else
							{
								;
							}
						}

						repeat = 1;//in case we have more commands in the buffer

					}
					else
					{
						Buffer_StoreElementOnBottom(&Rx_Buffer, 'r');//not enough data received yet
					}

					break;

				case 'w':
					if (Rx_Buffer.Elements >= 4)//ensure we have received enough data
						{
							Write_ISP_Byte(Buffer_GetElementFromBottom(&Rx_Buffer));
							Write_ISP_Byte(Buffer_GetElementFromBottom(&Rx_Buffer));
							Write_ISP_Byte(Buffer_GetElementFromBottom(&Rx_Buffer));
							Write_ISP_Byte(Buffer_GetElementFromBottom(&Rx_Buffer));
							repeat = 1;//in case we have more commands in the buffer
						}
						else
						{
							Buffer_StoreElementOnBottom(&Rx_Buffer, 'w');//not enough data received yet
						}

					break;
				case '3':
					if (Rx_Buffer.Elements >= 4)//ensure we have received enough data
					{
						Write_ISP_Byte(Buffer_GetElementFromBottom(&Rx_Buffer));
						Write_ISP_Byte(Buffer_GetElementFromBottom(&Rx_Buffer));
						Buffer_StoreElementOnTop(&Tx_Buffer, Write_Read_ISP_Byte(Buffer_GetElementFromBottom(&Rx_Buffer)) );
						Write_ISP_Byte(Buffer_GetElementFromBottom(&Rx_Buffer));
					}
					else
					{
						Buffer_StoreElementOnBottom(&Rx_Buffer, '3');//not enough data received yet
					}

					break;
				case '4':
					if (Rx_Buffer.Elements >= 4)//ensure we have received enough data
						{
							Write_ISP_Byte(Buffer_GetElementFromBottom(&Rx_Buffer));
							Write_ISP_Byte(Buffer_GetElementFromBottom(&Rx_Buffer));
							Write_ISP_Byte(Buffer_GetElementFromBottom(&Rx_Buffer));
							Buffer_StoreElementOnTop(&Tx_Buffer, Write_Read_ISP_Byte(Buffer_GetElementFromBottom(&Rx_Buffer)) );	
						}
						else
						{
							Buffer_StoreElementOnBottom(&Rx_Buffer, '4');//not enough data received yet
						}

					break;
				case 'b':

					Buffer_StoreElementOnTop(&Tx_Buffer, Wait_For_Busy());

					break;
				case 'e':

					DDRC &= ~( (1<<RES_2) | (1<<MOSI_2) | (1<<SCK_2) );

					Buffer_Initialize(&Rx_Buffer);//dump any unused data
					Buffer_Initialize(&Tx_Buffer);//dump any unused data

					Buffer_StoreElementOnTop(&Tx_Buffer, '\r');
					Buffer_StoreElementOnTop(&Tx_Buffer, '\n');
					Buffer_StoreElementOnTop(&Tx_Buffer, '>');
					mode = COMMAND;
					Scheduler_SetTaskMode(PROG_Task, TASK_STOP);		
					Scheduler_SetTaskMode(CDC_Task, TASK_RUN);

					break;

			}

		}	
	}while(repeat);

}


void ISP_Update_Buffers(void)
{
	if (USB_IsConnected)
	{
		/* Select the Serial Rx Endpoint */
		Endpoint_SelectEndpoint(CDC_RX_EPNUM);

		if (Endpoint_ReadWriteAllowed())
		{
			/* Read the recieved data endpoint into the transmission buffer */
			while (Endpoint_BytesInEndpoint())
			{
				Buffer_StoreElementOnTop(&Rx_Buffer, Endpoint_Read_Byte());
			}

			/* Clear the endpoint buffer */
			Endpoint_ClearCurrentBank();
		}
		
		/* Select the Serial Tx Endpoint */
		Endpoint_SelectEndpoint(CDC_TX_EPNUM);

		/* Write the Data to the Endpoint */
		if(Tx_Buffer.Elements != 0)
		{

			do{
				/* Wait until Serial Tx Endpoint Ready for Read/Write */
				while (!(Endpoint_ReadWriteAllowed()));

				Endpoint_Write_Byte(Buffer_GetElementFromBottom(&Tx_Buffer));

				/* Check if the endpoint is full */
				if (Endpoint_BytesInEndpoint() == CDC_TXRX_EPSIZE)
				{
					/* Send the data */
					Endpoint_ClearCurrentBank();

					/* Wait until Serial Tx Endpoint Ready for Read/Write */
					while (!(Endpoint_ReadWriteAllowed()));
				}
			}while (Tx_Buffer.Elements != 0);

			/* Send the last packet of the transfer to the host */
			Endpoint_ClearCurrentBank();
		}
	}
}


void ISP_Int(void)
{
	switch(mode)
	{
		case ISP1:
			break;
		case ISP2:

			Change_ISP_Pin(RES_2, HIGH);
			Change_ISP_Pin(MOSI_2, LOW);
			Change_ISP_Pin(SCK_2, LOW);
			//set my inputs/outputs
			DDRC &= ~(1<<MISO_2);
			DDRC |= (1<<MOSI_2) | (1<<SCK_2) | (1<<RES_2);
			
			break;
	}
}


//  Output MSB first
void  Write_ISP_Byte(unsigned char outbyte)
{

	unsigned char mask;
	unsigned char  i;

	Change_ISP_Pin(SCK_2,LOW);
//	DelayUsec(5+INTERBITDELAY);
	mask = 0x80;
	for(i=0; i<8; i++)
	{
		if((outbyte&mask))
		{
			Change_ISP_Pin(MOSI_2,HIGH);
		}
		else
		{
			Change_ISP_Pin(MOSI_2,LOW);
		}
	//	DelayUsec(INTERBITDELAY);
		Change_ISP_Pin(SCK_2,HIGH);
	//	DelayUsec(INTERBITDELAY);
		Change_ISP_Pin(SCK_2,LOW);
	//	DelayUsec(INTERBITDELAY);  // May not need this delay
		mask = mask>>1;
	}


	
}


unsigned char Write_Read_ISP_Byte(unsigned char outbyte)
{
	unsigned char mask,inbyte,i;

	Change_ISP_Pin(SCK_2,LOW);
//	DelayUsec(5+INTERBITDELAY);
	mask = 0x80;
	inbyte = 0x00;
	for(i=0; i<8; i++)
	{
		if((outbyte&mask))
		{
			Change_ISP_Pin(MOSI_2,HIGH);
		}
		else
		{
			Change_ISP_Pin(MOSI_2,LOW);
		}
	//	DelayUsec(10);
		Change_ISP_Pin(SCK_2,HIGH);
	//	DelayUsec(INTERBITDELAY);
		if(i>0)//??why check??
		{
			inbyte = inbyte<<1;
		}	
		if(PINC & (1<<MISO_2))
		{
			inbyte |= 0x01;
		}
	//	DelayUsec(INTERBITDELAY);
		Change_ISP_Pin(SCK_2,LOW);
	//	DelayUsec(INTERBITDELAY);  // May not need this delay
		mask = mask>>1;	
	}
	return inbyte;
}

unsigned char Wait_For_Busy(void)
{
	uint32_t loopcnt;

	loopcnt = 0;
	while((Chip_Busy_Status()&0x01))
	{
		DelayUsec(50);
		if(loopcnt++>5000)
		{
			return '1';
		}
	}
	return '0';
}
unsigned char Chip_Busy_Status(void)
{
	unsigned char busybyte;

	//  Read Fuse Low Byte 0
	Write_ISP_Byte(0xF0);//POLL_RDYBSY
	Write_ISP_Byte(0x00);
	Write_ISP_Byte(0x00);
	busybyte = Write_Read_ISP_Byte(0x00);

	return busybyte;
}





void Change_ISP_Pin(unsigned char pinId,unsigned char state)
{
	if(state == LOW)
	{
		PORTC &= ~(1<<pinId);
	}
	else
	{
		PORTC |= (1<<pinId);
	}

}


void DelayUsec(unsigned char delay)
{
	for(;delay; delay--)
	{
		;
	}

}
void uart_puts ( const char * s )
{
	while ( *s )
	{
		Buffer_StoreElementOnTop(&Rx_Buffer, *s++);
	}
	if(Transmitting == false)
	{
		Transmitting = true;
		Serial_TxByte(Buffer_GetElementFromBottom(&Rx_Buffer));
	}
}

ISR(USART1_TX_vect)
{
	/* Send next character if avaliable */
	if (Rx_Buffer.Elements != 0 && mode == SERIAL)
	{
		UDR1 = Buffer_GetElementFromBottom(&Rx_Buffer);
	}
	else
	{
		Transmitting = false;
	}
	  
}

//To play with
//ISR(USART1_UDRE_vect)//
//{
	/* Send next character if avaliable */
//	if (Rx_Buffer.Elements)
//	  UDR1 = Buffer_GetElement(&Rx_Buffer);
//	else
//	  Transmitting = false;

//	  UCSR1A = (1<<UDRE1);
//}

ISR(USART1_RX_vect)
{
	if(mode == SERIAL)
	{
		/* Character recieved, store it into the buffer */
		Buffer_StoreElementOnTop(&Tx_Buffer, UDR1);
	}
}

void ReconfigureUSART(void)
{
	uint8_t ConfigMask = 0;

	/* Determine parity - non odd/even parity mode defaults to no parity */
	if (LineCoding.ParityType == Parity_Odd)
	  ConfigMask = ((1 << UPM11) | (1 << UPM10));
	else if (LineCoding.ParityType == Parity_Even)
	  ConfigMask = (1 << UPM11);

	/* Determine stop bits - 1.5 stop bits is set as 1 stop bit due to hardware limitations */
	if (LineCoding.CharFormat == TwoStopBits)
	  ConfigMask |= (1 << USBS1);

	/* Determine data size - 5, 6, 7, or 8 bits are supported */
	if (LineCoding.DataBits == 6)
	  ConfigMask |= (1 << UCSZ10);
	else if (LineCoding.DataBits == 7)
	  ConfigMask |= (1 << UCSZ11);
	else if (LineCoding.DataBits == 8)
	  ConfigMask |= ((1 << UCSZ11) | (1 << UCSZ10));

	/* Enable double speed, gives better error percentages at 8MHz */
	UCSR1A = (1 << U2X1);

	/* Enable transmit and receive modules and interrupts */
	UCSR1B = ((1 << TXCIE1) | (1 << RXCIE1) | (1 << TXEN1) | (1 << RXEN1));//| (1 << UDRIE1)

	/* Set the USART mode to the mask generated by the Line Coding options */
	UCSR1C = ConfigMask;

	/* Set the USART baud rate register to the desired baud rate value */
	UBRR1  = MY_SERIAL_2X_UBBRVAL((uint16_t)LineCoding.BaudRateBPS);
}


uint8_t GetHexNumber(void)
{
	uint8_t temp, number;

	temp = Buffer_GetElementFromBottom(&Rx_Buffer);

	if(temp >= 'a' && temp <= 'f')
	{
		number = temp -'a' + 0xA;
	}
	else if(temp >= 'A' && temp <= 'F')
	{
		number = temp -'A' + 0xA;
	}
	else if(temp >= '0' && temp <= '9')
	{
		number = temp -'0';
	}
	else
	{
		Buffer_StoreElementOnBottom(&Rx_Buffer, temp);//if not a valid character return it to the buffer
		return 0;
	}

	temp = Buffer_GetElementFromBottom(&Rx_Buffer);


	number <<=4;//scale the number up 4 bits

	if(temp >= 'a' && temp <= 'f')
	{
		number += temp -'a' + 0xA;
	}
	else if(temp >= 'A' && temp <= 'F')
	{
		number += temp -'A' + 0xA;
	}
	else if(temp >= '0' && temp <= '9')
	{
		number += temp -'0';
	}
	else
	{
		Buffer_StoreElementOnBottom(&Rx_Buffer, temp);//if the character is no good to us, return it back to the buffer
	}


	return number;

}


uint8_t ParseNumber(void)
{
	uint8_t temp, number = 0;

	
	RemoveWhiteSpace();

	temp = Buffer_GetElementFromBottom(&Rx_Buffer);

	if(temp == '\\')
	{
		temp = Buffer_GetElementFromBottom(&Rx_Buffer);
		if(temp == 'x' || temp == 'X')
		{
			number = GetHexNumber();
		}
		else
		{
			Buffer_StoreElementOnBottom(&Rx_Buffer, temp);//if not a valid character return it to the buffer
			return 0;
		}	
	}
	else if(temp >= '0' && temp <= '9' )
	{
		do
		{
			number *= 10;
			number += temp - '0';
			temp = Buffer_GetElementFromBottom(&Rx_Buffer);
		} while (temp >= '0' && temp <= '9' );

		Buffer_StoreElementOnBottom(&Rx_Buffer, temp);//if not a valid character return it to the buffer
	}
	else//i.e. a character
	{
		number = temp;
	}
	

	return number;

}

void RemoveWhiteSpace(void)
{
	uint8_t temp;

	temp = Buffer_GetElementFromBottom(&Rx_Buffer);
	//remove white space
	while(temp == ' ' || temp == '\t' || temp == ',')
	{
		temp = Buffer_GetElementFromBottom(&Rx_Buffer);
	}
	Buffer_StoreElementOnBottom(&Rx_Buffer, temp);
}
